from __future__ import annotations
from typing import TYPE_CHECKING

import uno
from ooodev.events.args.event_args import EventArgs as EventArgs
from ooodev.adapter.adapter_base import AdapterBase, GenericArgs as GenericArgs

from com.sun.star.container import XContainerListener

if TYPE_CHECKING:
    from com.sun.star.lang import EventObject
    from com.sun.star.container import ContainerEvent


class ContainerListener(AdapterBase, XContainerListener):
    """
    receives events when the content of the related container changes.

    See Also:
        `API XContainerListener <https://api.libreoffice.org/docs/idl/ref/interfacecom_1_1sun_1_1star_1_1container_1_1XContainerListener.html>`_
    """

    def __init__(self, trigger_args: GenericArgs | None = None) -> None:
        """
        Constructor:

        Arguments:
            trigger_args (GenericArgs, optional): Args that are passed to events when they are triggered.
        """
        super().__init__(trigger_args=trigger_args)

    # region XContainerListener
    def elementInserted(self, event: ContainerEvent) -> None:
        """
        Event is invoked when a container has inserted an element.
        """
        self._trigger_event("elementInserted", event)

    def elementRemoved(self, event: ContainerEvent) -> None:
        """
        Event is invoked when a container has removed an element.
        """
        self._trigger_event("elementRemoved", event)

    def elementReplaced(self, event: ContainerEvent) -> None:
        """
        Event is invoked when a container has replaced an element.
        """
        self._trigger_event("elementReplaced", event)

    def disposing(self, event: EventObject) -> None:
        """
        Gets called when the broadcaster is about to be disposed.

        All listeners and all other objects, which reference the broadcaster
        should release the reference to the source. No method should be invoked
        anymore on this object ( including ``XComponent.removeEventListener()`` ).

        This method is called for every listener registration of derived listener
        interfaced, not only for registrations at ``XComponent``.
        """
        # from com.sun.star.lang.XEventListener
        self._trigger_event("disposing", event)

    # endregion XContainerListener
